@@ -36,7 +36,7 @@ except ImportError:
     )
 
 from rest_framework.request import Request
-from rest_framework.serializers import Serializer
+from rest_framework.serializers import BaseSerializer
 
 
 if TYPE_CHECKING:
@@ -44,10 +44,9 @@ if TYPE_CHECKING:
 
 
 __all__ = [
-    "MethodType",
+    "HTTPMethod",
     "LogicCallable",
     "PipelineLogic",
-    "PipelineDefinition",
     "PipelinesDict",
     "ViewContext",
     "TYPE_CHECKING",
@@ -67,16 +66,19 @@ __all__ = [
     "DataDict",
     "DataConditional",
     "DataReturn",
+    "SerializerType",
+    "TypesDict",
 ]
 
 DataDict = Dict[str, Any]
+SerializerType = Type[BaseSerializer]
 DataConditional = Tuple[Any, DataDict]
 DataReturn = Union[DataDict, DataConditional]
-MethodType = Literal["GET", "POST", "PUT", "PATCH", "DELETE"]
+HTTPMethod = Literal["GET", "POST", "PUT", "PATCH", "DELETE"]
 LogicCallable = Callable[..., Optional[DataReturn]]
-PipelineLogic = Union[LogicCallable, Iterable["PipelineLogic"]]
-PipelineDefinition = Iterable[Union[PipelineLogic, Type[Serializer]]]
-PipelinesDict = Dict[MethodType, PipelineDefinition]
+PipelineLogic = Union[LogicCallable, SerializerType, Iterable["PipelineLogic"]]
+PipelinesDict = Dict[HTTPMethod, PipelineLogic]
+TypesDict = Dict[str, Union[Optional[Type], "TypesDict"]]
 
 
 class ViewContext(TypedDict):
